# !mkdir images
# %cd images

#from google.colab import files
from io import BytesIO
import torch
import sys
from fastai.vision.all import *
from torchvision.utils import save_image #, load_image
#from fastai.vision.all import unet_learner, DataLoaders
import matplotlib.pyplot as plt
import numpy as np
import subprocess
import cv2
from dagshub import dagshub_logger, DAGsHubLogger


class ImageImageDataLoaders(DataLoaders):
    "Basic wrapper around several `DataLoader`s with factory methods for Image to Image problems"
    @classmethod
    @delegates(DataLoaders.from_dblock)
    def from_label_func(cls, path, fnames, label_func, valid_pct=0.2, seed=None, item_tfms=None, batch_tfms=None, **kwargs):
        "Create from list of `fnames` in `path`s with `label_func`."
        dblock = DataBlock(blocks=(ImageBlock(cls=PILImage), ImageBlock(cls=PILImageBW)),
                           splitter=RandomSplitter(valid_pct, seed=seed),
                           get_y=label_func,
                           item_tfms=item_tfms,
                           batch_tfms=batch_tfms)
        res = cls.from_dblock(dblock, fnames, path=path, **kwargs)
        return res

def get_y_fn(x):
    y = str(x.absolute()).replace('.jpg', '_depth.png')
    y = Path(y)
    return y

def create_data(data_path):
    fnames = get_files(data_path/'test', extensions='.jpg')
    data = ImageImageDataLoaders.from_label_func(data_path/'test', seed=42, bs=4, num_workers=0, fnames=fnames) #, label_func=get_y_fn)
    return data

# e.g. python3 testing.py ../data/processed

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print("usage: %s <data_path> <display_thresh>" % sys.argv[0], file=sys.stderr)
        sys.exit(0)

    corr_disp_thresh = float(sys.argv[2])
    data_path = Path(sys.argv[1])
    #data = create_data(data_path)
    dblock = DataBlock(blocks = (ImageBlock(cls=PILImage),ImageBlock(cls=PILImageBW)))
    fnames = get_files(data_path/'test', extensions='.jpg')
    #data = ImageDataLoaders.from_lists(data_path, [Path(fnames[0]),], [Path(str(fnames[0]).replace('.jpg', '_depth.png')),], bs=1)
    data = ImageImageDataLoaders.from_dblock(dblock, [Path(fnames[0]),], bs=1)
    learner = unet_learner(data, resnet34, n_out=3, pretrained_model=None, loss_func=MSELossFlat(), path='../')
    learner = learner.load('model')
    logger = DAGsHubLogger('../eval/depth_map_corr.csv')

    logger.log_hyperparams({'model' : 'model'})
    for i, fn in enumerate(fnames):
      res = learner.predict(fn)
      cvimg = 256*np.uint16(res[0].numpy().transpose(1, 2, 0))
      gray = cv2.cvtColor(cvimg, cv2.COLOR_BGR2GRAY)
      depth_img = cv2.imread(str(fn).replace('.jpg','_depth.png'))
      gray_depth_img = cv2.cvtColor(depth_img, cv2.COLOR_BGR2GRAY)
      corr = np.sum(cv2.multiply(gray - cv2.mean(gray)[0], gray_depth_img - cv2.mean(gray_depth_img)[0])) / cv2.norm(gray - cv2.mean(gray)[0]) / cv2.norm(gray_depth_img - cv2.mean(gray_depth_img)[0])
      print(i,fn,corr)
      logger.log_metrics(norm_corr_diff=corr, img=fn)
      if corr<corr_disp_thresh:
        cv2.namedWindow('predicted depth', cv2.WINDOW_AUTOSIZE)
        cv2.imshow('predicted depth', gray)
        cv2.namedWindow('ref depth', cv2.WINDOW_AUTOSIZE)
        cv2.imshow('ref depth', depth_img)
        cv2.waitKey(-1)
      logger.save()
    logger.close()

